@@ -21,7 +21,8 @@
 #endif
 #include <libHX.h>
 #include <libHX/libxml_helper.h>
-#include <pcre.h>
+#define PCRE2_CODE_UNIT_WIDTH 8
+#include <pcre2.h>
 #include "libcryptmount.h"
 #include "pam_mount.h"
 
@@ -71,8 +72,8 @@ struct pmt_command {
 static int rc_volume_cond_ext(const struct passwd *, xmlNode *);
 
 /* Variables */
-static const struct callbackmap cf_tags[];
-static const struct pmt_command default_command[];
+static const struct callbackmap cf_tags[26];
+static const struct pmt_command default_command[20];
 
 //-----------------------------------------------------------------------------
 /**
@@ -139,7 +140,7 @@ static bool expand_user(const char *user, char **dest_pptr,
  * Like, turning <volume server="w2k3" path="%(USER)"
  * mountpoint="~" /> into path="joe" mountpoint="/home/joe".
  */
-bool expandconfig(const struct config *config)
+bool expandconfig(struct config *config)
 {
 	const char *u = config->user;
 	struct HXformat_map *vinfo;
@@ -171,6 +172,18 @@ bool expandconfig(const struct config *config)
 	format_add(vinfo, "GROUP", (ge != NULL) ? ge->gr_name : "");
 	misc_add_ntdom(vinfo, u);
 
+	if (config->luserconf != NULL) {
+		char *tmp = NULL;
+		tmp = xstrdup(config->luserconf);
+		if (!expand_home(u, &tmp) || !expand_user(u, &tmp, vinfo)) {
+			free(tmp);
+			goto rfalse;
+		}
+		HXmc_free(config->luserconf);
+		config->luserconf = HXmc_strinit(tmp);
+		free(tmp);
+	}
+
 	HXlist_for_each_entry(vpt, &config->volume_list, list) {
 		if (vpt->is_expanded)
 			continue;
@@ -576,25 +589,28 @@ static inline bool parse_bool_f(char *s)
 static int pmt_strregmatch(const char *s, const char *pattern, bool icase)
 {
 	/* Mostly compile-time flags that are not valid for pcre_exec */
-	unsigned int flags = PCRE_DOLLAR_ENDONLY | PCRE_DOTALL |
-	                     PCRE_NO_AUTO_CAPTURE;
-	const char *error = NULL;
-	int erroffset, ret;
-	pcre *rd;
+	unsigned int flags = PCRE2_DOLLAR_ENDONLY | PCRE2_DOTALL |
+	                     PCRE2_NO_AUTO_CAPTURE;
+	int errcode = 0, ret;
+	PCRE2_SIZE erroffset;
+	pcre2_code_8 *rd;
+	PCRE2_UCHAR buffer[256];
+	pcre2_match_data *match_data;
 
 	if (icase)
-		flags |= PCRE_CASELESS;
-	rd = pcre_compile(pattern, flags, &error, &erroffset, NULL);
-	if (error != NULL) {
-		l0g("pcre_compile failed: %s at offset %d\n", error, erroffset);
-		return -1;
-	} else if (rd == NULL) {
-		l0g("pcre_compile failed: %s\n", strerror(errno));
+		flags |= PCRE2_CASELESS;
+	rd = pcre2_compile(reinterpret_cast(PCRE2_SPTR, pattern),
+	     PCRE2_ZERO_TERMINATED, flags, &errcode, &erroffset, NULL);
+	if (rd == NULL) {
+		l0g("pcre2_compile failed: %s at offset %d\n",
+		    buffer, static_cast(int, erroffset));
 		return -1;
 	}
 
-	ret = pcre_exec(rd, NULL, s, strlen(s), 0, 0, NULL, 0);
-	if (ret == PCRE_ERROR_NOMATCH) {
+	match_data = pcre2_match_data_create_from_pattern(rd, NULL);
+	ret = pcre2_match(rd, reinterpret_cast(PCRE2_SPTR, s), strlen(s),
+	      0, 0, match_data, 0);
+	if (ret == PCRE2_ERROR_NOMATCH) {
 		l0g("pcre_exec: no match\n");
 		ret = false;
 	} else if (ret < 0) {
@@ -604,7 +620,8 @@ static int pmt_strregmatch(const char *s, const char *pattern, bool icase)
 		ret = true;
 		l0g("pcre_exec: /%s/: %d matches\n", pattern, ret);
 	}
-	pcre_free(rd);
+	pcre2_match_data_free(match_data);
+	pcre2_code_free(rd);
 	return ret;
 }
 
@@ -750,8 +767,11 @@ static const char *rc_luserconf(xmlNode *node, struct config *config,
 	if ((s = xml_getprop(node, "name")) == NULL)
 		return "<luserconf> is missing name= attribute";
 	HXmc_free(config->luserconf);
-	config->luserconf = HXmc_strinit(pent->pw_dir);
-	HXmc_strcat(&config->luserconf, "/");
+	config->luserconf = HXmc_strinit("");
+	if (strlen(s) < 1 || s[0] != '/') {
+		HXmc_strcat(&config->luserconf, pent->pw_dir);
+		HXmc_strcat(&config->luserconf, "/");
+	}
 	HXmc_strcat(&config->luserconf, s);
 	w4rn("path to luserconf set to %s\n", config->luserconf);
 	free(s);
@@ -956,7 +976,7 @@ static int rc_volume_cond_xor(const struct passwd *pwd, xmlNode *node)
 static int rc_volume_cond_not(const struct passwd *pwd, xmlNode *node)
 {
 	unsigned int count = 0;
-	bool ret = true;
+	int ret = true;
 
 	for (node = node->children; node != NULL; node = node->next) {
 		if (node->type != XML_ELEMENT_NODE)
@@ -1448,7 +1468,7 @@ static const struct pmt_command default_command[] = {
 	{CMD_UMOUNT,     NULL,     {"umount", "%(MNTPT)", NULL}},
 	{CMD_FSCK,       NULL,     {"fsck", "-p", "%(FSCKTARGET)", NULL}},
 	{CMD_PMVARRUN,   NULL,     {"pmvarrun", "-u", "%(USER)", "-o", "%(OPERATION)", NULL}},
-	{CMD_FD0SSH,      NULL,    {"pmt-fd0ssh", NULL}},
+	{CMD_FD0SSH,      NULL,    {"fd0ssh", NULL}},
 	{CMD_OFL,        NULL,     {"ofl", "-k%(SIGNAL)", "%(MNTPT)", NULL}},
 	{-1},
 };
